module Pages.HomeView exposing (homeView)

import Types exposing (Model, Msg(..))
import Html exposing (Html, div, p, a, text, form, input, button, h1)
import Html.Attributes exposing (class, type_, placeholder, value, href)
import Html.Events exposing (onSubmit, onClick, onInput)
import String exposing (startsWith)

-- Function to render the home view, allowing the user to enter a Quiz ID and start a quiz.
homeView : Model -> Html Msg
homeView model = 
    let
        -- Determine the loading message based on the current error state.
        loadingMsg = if model.error == "404" then
                p [ class "error" ] [ text "Diese Quiz ID gibt es nicht! Bitte gib eine andere ein."]
            else if startsWith "Error" model.error then
                p [ class "danger" ] [ text model.quizId ]
            else
                p [] [ text "zum testen: y9sM9sY9gb9hVqnOX4Zc, mwPpDw0HlJ1l65WObssa" ]
    in
    -- Main container div with a background and centered content.
    div [ class "home" ] 
        [ div [ class "background" ] []
        , div [ class "center" ]
            [ -- Title of the home page.
              h1 [ class "title" ] [ text "Mathe Quizzes" ]
              -- Form to submit the Quiz ID.
            , form [ onSubmit SendQuizRequest ] 
                [ -- Input field for the Quiz ID.
                  input
                    [ type_ "text"
                    , placeholder "Quiz ID"
                    , value model.quizId
                    , onInput UpdateQuizId
                    , class "input-field" ]
                    []
                    -- Submit button for the form.
                , button
                    [ onClick SendQuizRequest
                    , type_ "submit"
                    , class "submit-button" ]
                    [ text "Öffnen" ]
                ]
              -- Display the loading message or any error message.
            , loadingMsg 
            , a [ class "link", href "/editor" ] [ text "Quiz Editor" ]
            ] 
        ]