port module QuizEditor.Editor exposing (main)

import Browser
import Html exposing (Html, div, text, button, p, br)
import Html.Events exposing (onClick)
import Json.Decode as Decode
import Json.Encode as Encode
import Types exposing (PartialQuiz)
import Utils.Api exposing (partialQuizDecoder)


port resetInput : () -> Cmd msg

type alias Model =
    { result : Result Decode.Error PartialQuiz }

type Msg
    = ResetInput

main : Program Decode.Value Model Msg
main =
    Browser.element
        { init = \flags -> init flags
        , update = update
        , subscriptions = \_ -> Sub.none
        , view = view
        }

init : Decode.Value -> ( Model, Cmd Msg )
init flags =
    let
        result =
            Decode.decodeValue partialQuizDecoder flags
    in
    ( Model result, Cmd.none )


update : Msg -> Model -> ( Model, Cmd Msg )
update msg model =
    case msg of
        ResetInput ->
            ( model, resetInput () )


view : Model -> Html Msg
view model =
    div []
        [ viewResult model.result ]


viewResult : Result Decode.Error PartialQuiz -> Html Msg
viewResult result =
    let
        resulttext = case result of
            Ok _ ->
                p [] [ text "Valid JSON" ]

            Err error ->
                div []
                    [ p [] [ text (formatDecodeError error) ] 
                    , br [] []
                    , p [] [ text "complete Error message:" ]
                    , p [] [ text (Decode.errorToString error) ]
                    ]
    in
    div []
        [ resulttext
        , button [ onClick (ResetInput )] [ text "reset" ]]
    
formatDecodeError : Decode.Error -> String
formatDecodeError error =
    case error of
        Decode.Field field subError ->
            "Field error at \"" ++ field ++ "\": " ++ formatDecodeError subError

        Decode.Index index subError ->
            "Index error at [" ++ String.fromInt index ++ "]: " ++ formatDecodeError subError

        Decode.OneOf errors ->
            "One of the following errors occurred:\n" ++ String.join "\n" (List.map formatDecodeError errors)

        Decode.Failure msg json ->
            "Failure: " ++ msg ++ "\n" ++ "JSON: " ++ Encode.encode 2 json
