module Utils.HelperFunction exposing (..)

import Types exposing (..)
import Utils.Api exposing (..)

import Url
import Url.Parser exposing ((</>), int)

import Html exposing (Html, li, a, text)
import Html.Attributes exposing (href)

import String exposing (toLower, replace)

-- Initialize an empty quiz.
initQuiz : Quiz 
initQuiz =
    Quiz "" "" Nothing []

-- URL parser for routing within the application.
urlParser : Url.Parser.Parser (Route -> a) a
urlParser =
    Url.Parser.oneOf
        [ Url.Parser.map HomeRoute <| Url.Parser.top
        , Url.Parser.map HomeRoute <| Url.Parser.s "home"
        , Url.Parser.map QuestionRoute <| Url.Parser.s "question" </> int
        , Url.Parser.map ResultRoute <| Url.Parser.s "result"
        ]

-- Extract the route from a given URL.
getRoute : Url.Url -> Route
getRoute url =
    Maybe.withDefault NotFound (Url.Parser.parse urlParser url)

-- Create a view link element.
viewLink : String -> Html msg
viewLink path =
    li [] [ a [ href path ] [ text path ] ]

-- Normalize a string by converting it to lowercase and removing spaces.
normalizeString : String -> String
normalizeString str =
    str
        |> toLower
        |> replace " " ""

-- Retrieve a question at a specific index from the list of questions.
getQuestionAt : Int -> List Question -> Maybe Question
getQuestionAt index q =
    case q of 
        [] -> Nothing

        x :: xs ->
            if index > 0 then
                getQuestionAt (index - 1) xs
            else
                Just x

-- Update a question at a specific index with a new value.
updateQuestionsAt : Int -> String -> List Question -> List Question
updateQuestionsAt index newValue q =
    case q of 
        [] -> []

        x :: xs -> 
            if index > 0 then
                x :: updateQuestionsAt (index - 1) newValue xs
            else
                let 
                    newQuestionType = case x.questionType of
                        MCQ a -> 
                            let
                                -- Update the user selection for MCQ (Multiple Choice Question).
                                newUserSelection = if List.member newValue a.userSelection 
                                    then List.filter (\o -> o /= newValue) a.userSelection
                                    else a.userSelection ++ [newValue]
                            in
                            MCQ { a | userSelection = newUserSelection }
                        SCQ a -> SCQ { a | userSelection = newValue } -- Update the user selection for SCQ (Single Choice Question).
                        Input a -> Input { a | userSelection = newValue } -- Update the user selection for Input type question.
                    newQuestion = {x | questionType = newQuestionType}
                in
                    newQuestion :: xs
