module Utils.MathFunction exposing (..)

import Types exposing (MathFunction(..))

toLambda : MathFunction -> (Float -> Float)
toLambda mathFunction =
    case mathFunction of
        Const x ->
            \_ -> x

        X ->
            \x -> x

        Add f1 f2 ->
            let
                lf1 = toLambda f1
                lf2 = toLambda f2
            in
            \x -> (lf1 x) + (lf2 x)

        Sub f1 f2 ->
            let
                lf1 = toLambda f1
                lf2 = toLambda f2
            in
            \x -> (lf1 x) - (lf2 x)

        Mul f1 f2 ->
            let
                lf1 = toLambda f1
                lf2 = toLambda f2
            in
            \x -> (lf1 x) * (lf2 x)

        Div f1 f2 ->
            let
                lf1 = toLambda f1
                lf2 = toLambda f2
            in
            \x -> (lf1 x) / (lf2 x)

        Pow f1 f2 ->
            let
                lf1 = toLambda f1
                lf2 = toLambda f2
            in
            \x -> (lf1 x) ^ (lf2 x)

        Log f1 f2 ->
            let
                lf1 = toLambda f1
                lf2 = toLambda f2
            in
            \x -> Basics.logBase (lf1 x) (lf2 x)

        Ln f ->
            let
                lf = toLambda f
            in
            \x -> Basics.logBase Basics.e (lf x)

        Neg f ->
            let
                lf = toLambda f
            in
            \x -> -1 * (lf x)
            

        Sin f ->
            let
                lf = toLambda f
            in
            \x -> Basics.sin (lf x)

        Cos f ->
            let
                lf = toLambda f
            in
            \x -> Basics.sin (lf x)

        Tan f ->
            let
                lf = toLambda f
            in
            \x -> Basics.tan (lf x)

        Exp f ->
            let
                lf = toLambda f
            in
            \x -> Basics.e ^ (lf x)

toString : MathFunction -> String
toString mathFunction =
    case mathFunction of
        Const x ->
            String.fromFloat x

        X ->
            "x"

        Add f1 f2 ->
            let
                lf1 = toString f1
                lf2 = toString f2
            in
            "( " ++ lf1 ++ " ) + ( " ++ lf2 ++ " )"

        Sub f1 f2 ->
            let
                lf1 = toString f1
                lf2 = toString f2
            in
            "( " ++ lf1 ++ " ) - ( " ++ lf2 ++ " )"

        Mul f1 f2 ->
            let
                lf1 = toString f1
                lf2 = toString f2
            in
            "( " ++ lf1 ++ " ) * ( " ++ lf2 ++ " )"

        Div f1 f2 ->
            let
                lf1 = toString f1
                lf2 = toString f2
            in
            "( " ++ lf1 ++ " ) / ( " ++ lf2 ++ " )"

        Pow f1 f2 ->
            let
                lf1 = toString f1
                lf2 = toString f2
            in
            "( " ++ lf1 ++ " ) ^ ( " ++ lf2 ++ " )"

        Log f1 f2 ->
            let
                lf1 = toString f1
                lf2 = toString f2
            in
            "log_{ " ++ lf1 ++ " }( " ++ lf2 ++ " )"

        Ln f ->
            let
                lf = toString f
            in
            "ln( " ++ lf ++ " )"

        Neg f ->
            let
                lf = toString f
            in
            "- ( " ++ lf ++ " )"
            

        Sin f ->
            let
                lf = toString f
            in
            "sin( " ++ lf ++ " )"

        Cos f ->
            let
                lf = toString f
            in
            "cos( " ++ lf ++ " )"

        Tan f ->
            let
                lf = toString f
            in
            "tan( " ++ lf ++ " )"

        Exp f ->
            let
                lf = toString f
            in
            "e^( " ++ lf ++ " )"